/*
 * This file is part of the digitalSTROM ds485-core library.
 *
 * Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland
 *
 * digitalSTROM ds485-core library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation version 2
 * of the License.
 *
 * digitalSTROM ds485-core library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with digitalSTROM ds485-core library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/*
 * intermediate file, till we depend on libsduid library
 * move all dsuid_t related functions into one place
 */

#include "dsuid_helper.h"

#include "digitalSTROM/ds.h"
#include "digitalSTROM/ds485-const.h"

dsuid_t NO_DEVICE    = { { 0 } };

/*
 * global list of dsuid
 */
static dsuid_t s_busDsuids[DS485_MAX_SLAVE_ADDRESS + 1];

dsuid_t ds485_dsid_to_dsuid(const dsid_t *dsid)
{
    dsuid_t dsuid;

    dsuid.id[0] = dsid->id[0];
    dsuid.id[1] = dsid->id[1];
    dsuid.id[2] = dsid->id[2];
    dsuid.id[3] = dsid->id[3];
    dsuid.id[4] = dsid->id[4];
    dsuid.id[5] = dsid->id[5];
    dsuid.id[6] = 0;
    dsuid.id[7] = 0;
    dsuid.id[8] = 0;
    dsuid.id[9] = 0;
    dsuid.id[10] = dsid->id[6];
    dsuid.id[11] = dsid->id[7];
    dsuid.id[12] = dsid->id[8];
    dsuid.id[13] = dsid->id[9];
    dsuid.id[14] = dsid->id[10];
    dsuid.id[15] = dsid->id[11];
    dsuid.id[16] = 0;

    return dsuid;
}

void dsuidToString(const dsuid_t *dsuid, char *dsuid_str)
{
    char *p;
    uint8_t i;
    p = dsuid_str;
    for (i = 0; i < sizeof(dsuid_t); ++i) {
        p += sprintf(p, "%02x", dsuid->id[i]);
    }
    dsuid_str[DSUID_STR_LEN - 1] = '\0';
}

static int cmpAddress(uint8_t addr, const dsuid_t* dsuid)
{
    if (addr > DS485_MAX_SLAVE_ADDRESS) {
        return false;
    }
    return memcmp(&s_busDsuids[addr], dsuid, sizeof(dsuid_t));
}

bool isAddressValid(uint8_t addr)
{
    return cmpAddress(addr, &NO_DEVICE) != 0;
}

const dsuid_t *get_dsuid(uint8_t addr)
{
    if (addr > DS485_MAX_SLAVE_ADDRESS) {
        return &NO_DEVICE;
    }
    return &s_busDsuids[addr];
}
void set_dsuid(uint8_t addr, const dsuid_t *dsuid)
{
    if (addr > DS485_MAX_SLAVE_ADDRESS) {
        /* ignore, will not be able to lookup */
    }
    s_busDsuids[addr] = *dsuid;
}

uint8_t get_short_address(const dsuid_t *dsuid)
{
    int i;
    for (i = 0; i <= DS485_MAX_SLAVE_ADDRESS; i++) {
        /* memcmp returns 0 if it matches */
        if (cmpAddress(i, dsuid) == 0) {
            return i;
        }
    }
    return DS485_INVALID_ADDRESS;
}

void copy_dsuids(dsuid_t *list, int size)
{
    if (size > sizeof(s_busDsuids)) {
        size = sizeof(s_busDsuids);
    }
    memcpy(list, s_busDsuids, size);
}
